package com.bobardo.bazaar.iab;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

import com.bobardo.bazaar.iab.util.IabHelper;
import com.bobardo.bazaar.iab.util.IabResult;
import com.bobardo.bazaar.iab.util.Inventory;
import com.bobardo.bazaar.iab.util.Purchase;
import com.bobardo.bazaar.iab.util.IabHelper.OnConsumeFinishedListener;
import com.bobardo.bazaar.iab.util.IabHelper.OnIabPurchaseFinishedListener;
import com.bobardo.bazaar.iab.util.IabHelper.QueryInventoryFinishedListener;
import com.unity3d.player.UnityPlayer;

/* Apache License. Copyright (C) Bobardo Studio - All Rights Reserved.
 * Unauthorized publishing the plugin with different name is strictly prohibited.
 * This plugin is free and no one has right to sell it to others.
 * http://bobardo.com
 * http://opensource.org/licenses/Apache-2.0
 */

public class ServiceBillingBazaar {

	private IabHelper mHelper;
	private static ServiceBillingBazaar instance;
	private static Context context;
	
	private static String PUBLIC_KEY = null;
	private static final int RC_REQUEST = 10001;
	
	private static final String SKU = "ID#sku";
	private static final String EXTRA_DATA = "ID#extraData";
	private static final String OPERATION_TYPE = "ID#operationType";
	private static final String TAG = "BazaarIabService";
	private static final int PURCHASE = 1;
	private static final int HAS_PURCHASE = 2;
	private static final int PURCHASE_CONSUME = 3;
	
	private static final String GameObjectName = "com.bobardo IAB";
	
	public ServiceBillingBazaar() {
		ServiceBillingBazaar.instance = this;
	}

	public static void setContext(Context context){
		ServiceBillingBazaar.context = context;
	}
	
	public static ServiceBillingBazaar instance() {
		if(instance == null) {
			instance = new ServiceBillingBazaar();
		}
		return instance;
	}

	public void setPublicKey(String publicKey){
		PUBLIC_KEY = publicKey;
	}
	
	public void startIabServiceInBg() {
		if(PUBLIC_KEY == null){
			String msg = "error!!! PublicKey base64Encoded is NULL";
			log(msg);
			UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(15, msg));
			return;
		}
		startIabHelper();
	}

	private synchronized void startIabHelper(){

		String base64EncodedPublicKey = PUBLIC_KEY;
		try {

			mHelper = new IabHelper(context, base64EncodedPublicKey);

			mHelper.startSetup(new IabHelper.OnIabSetupFinishedListener() {

				public void onIabSetupFinished(IabResult result) {

					if (!result.isSuccess()) {

						String msg = "(startIabHelper) error setup bazaar billing. " + result.getMessage();
						log(msg);
						UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(12, msg));
						return;
					}

					if (mHelper == null) {
						String msg = "(startIabHelper) error setup bazaar billing. iabHelper is null.";
						log(msg);
						UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(13, msg));
						return;
					}

				}
			});
		} catch (Exception e) {
			String msg = "(startIabHelper) error setup bazaar billing. " + e.getMessage();
			log(msg);
			UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(14, msg));
		}

	}
	
	public void stopIabHelper(){
		if (mHelper != null) {
			try {
				mHelper.dispose();
				mHelper = null;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	public void launchPurchaseFlow(String sku, String extraData) {

		try {
			final Intent intent = new Intent(context, IabActivity.class);
			intent.putExtra(OPERATION_TYPE, PURCHASE);
			intent.putExtra(SKU, sku);
			intent.putExtra(EXTRA_DATA, extraData);

			context.startActivity(intent);

		} catch(Exception e){
			String msg = "(launchPurchaseFlow) Error purchasing item. " + e.getMessage();
			log(msg);
			UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(1, msg));
		}

	}
	
	public void PurchaseAndConsume(String sku, String extraData) {

		try {
			final Intent intent = new Intent(context, IabActivity.class);
			intent.putExtra(OPERATION_TYPE, PURCHASE_CONSUME);
			intent.putExtra(SKU, sku);
			intent.putExtra(EXTRA_DATA, extraData);

			context.startActivity(intent);

		} catch(Exception e){
			String msg = "(PurchaseAndConsume) Error purchasing item. " + e.getMessage();
			log(msg);
			UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(2, msg));
		}

	}

	public void checkHasPurchase(String sku){
		try {
			
			final Intent intent = new Intent(context, IabActivity.class);
			intent.putExtra(OPERATION_TYPE, HAS_PURCHASE);
			intent.putExtra(SKU, sku);

			context.startActivity(intent);

		} catch(Exception e){
			String msg = "(checkHasPurchase) Error checking item. " + e.getMessage();
			log(msg);
			UnityPlayer.UnitySendMessage(GameObjectName, "getInventoryResult", returnResult(3, msg));
		}
	}

	public static class IabActivity extends Activity {

		private String productId, payload;
		private boolean consume = false;
		
		@Override
		protected void onCreate(Bundle savedInstanceState) {
			super.onCreate(savedInstanceState);

			if(savedInstanceState != null) {
				productId = savedInstanceState.getString("productId");
				payload = savedInstanceState.getString("payload");
				consume = savedInstanceState.getBoolean("consume");
			}
			else 
			{
				Intent intent = getIntent();
	
				productId = intent.getStringExtra(SKU);
				int operationType = intent.getIntExtra(OPERATION_TYPE, 0);
	
				switch (operationType) {
				case PURCHASE:
					payload = intent.getStringExtra(EXTRA_DATA);
					purchaseProduct();
					break;
				case HAS_PURCHASE:
					hasPurchase();
					break;
				case PURCHASE_CONSUME:
					payload = intent.getStringExtra(EXTRA_DATA);
					purchaseProduct();
					consume = true;
					break;
				default:
					finish();	
					break;
				}
			}
		}

		@Override
		public void onSaveInstanceState(Bundle savedInstanceState) {
		  super.onSaveInstanceState(savedInstanceState);

		  savedInstanceState.putString("productId", productId);
		  savedInstanceState.putString("payload", payload);
		  savedInstanceState.putBoolean("consume", consume);
		}
		
		private void purchaseProduct(){
			try {
				
				ServiceBillingBazaar.instance().mHelper.launchPurchaseFlow(this, productId, RC_REQUEST, iabPurchaseFinishedListener, payload);
				
			} catch (IllegalStateException e) {
				String msg = "Error purchasing item. " + e.getMessage();
				log(msg);
				UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(4, msg));
				finish();
			} catch (Exception e) {
				String msg = "Error purchasing item. " + e.getMessage();
				log(msg);
				UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(5, msg));
				finish();
			}
		}
		
		private void hasPurchase(){
			try {
				ServiceBillingBazaar.instance().mHelper.queryInventoryAsync(mGotInventoryListener);
			} catch (IllegalStateException e) {
				String msg = "Error purchasing item. " + e.getMessage();
				log(msg);
				UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(4, msg));
				finish();
			} catch (Exception e) {
				String msg = "Error purchasing item. " + e.getMessage();
				log(msg);
				UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(5, msg));
				finish();
			}
		}
		
		@Override
		protected void onActivityResult(int requestCode, int resultCode, Intent data) {
			if (ServiceBillingBazaar.instance().mHelper != null && !ServiceBillingBazaar.instance().mHelper.handleActivityResult(requestCode, resultCode, data)) {
				super.onActivityResult(requestCode, resultCode, data);
			}
			finish();
		}

		private OnIabPurchaseFinishedListener iabPurchaseFinishedListener = new OnIabPurchaseFinishedListener() {

			@Override
			public void onIabPurchaseFinished(IabResult result, Purchase info) {

				log("IabPurchase finished: " + result + ", purchase: " + info);

				if (result.getResponse() == IabHelper.BILLING_RESPONSE_RESULT_OK) {
					
					log("purchase ok");
					if (!info.getSku().equals(productId)){
						String msg = "purchase wrong product!!! purchased product: " + info.getSku();
						log(msg);
						UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(6, msg));
						return;
					}
					
					if(consume){
						ServiceBillingBazaar.instance().mHelper.consumeAsync(info, mConsumeFinishedListener);
					}else{
						String msg = info.toString().replace("PurchaseInfo(type:inapp):", "");
						UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(0, msg));						
					}
					
				} else if (result.getResponse() == IabHelper.BILLING_RESPONSE_RESULT_USER_CANCELED) {
					String msg = "user canceled the purchase";
					log(msg);
					UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(6, msg));

				} else if (result.getResponse() == IabHelper.BILLING_RESPONSE_RESULT_ITEM_ALREADY_OWNED) {
					String msg = "user already has this product";
					log(msg);
					
					if(consume){
						ServiceBillingBazaar.instance().mHelper.consumeAsync(info, mConsumeFinishedListener);
					}else{
						msg = info.toString().replace("PurchaseInfo(type:inapp):", "");
						log(msg);
						UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(0, msg));						
					}
					
				} else {
					String msg = "failed to purchase";
					log(msg);
					UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(7, msg));

				}
				
			}
		};
		
		private OnConsumeFinishedListener mConsumeFinishedListener = new IabHelper.OnConsumeFinishedListener() {
			public void onConsumeFinished(Purchase purchase, IabResult result) {
				if (result.isSuccess()) {
					// provision the in-app purchase to the user
					// (for example, credit 50 gold coins to player's character)
					String msg = purchase.toString().replace("PurchaseInfo(type:inapp):", "");
					log(msg);
					UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(0, msg));						

				}
				else {
					// handle error
					String msg = "failed to consume product. but the purchase was successful. result: " + result.getMessage();
					log(msg);
					UnityPlayer.UnitySendMessage(GameObjectName, "getPurchaseResult", returnResult(8, msg));
				}
			}
		};
		
		QueryInventoryFinishedListener mGotInventoryListener = new IabHelper.QueryInventoryFinishedListener() {
			public void onQueryInventoryFinished(IabResult result, Inventory inventory) {
				
				if (result.getResponse()==6) {

					String msg = "user should login to cafebazaar.";
					log(msg);
					Toast.makeText(IabActivity.this, "ابتدا باید وارد حساب کافه بازار شوید", Toast.LENGTH_SHORT).show();
					UnityPlayer.UnitySendMessage(GameObjectName, "getInventoryResult", returnResult(9, msg));
					finish();
					return;

				}
				
				if (result.isFailure()) {
					String msg = "error connecting cafebazaar.";
					log(msg);
					UnityPlayer.UnitySendMessage(GameObjectName, "getInventoryResult", returnResult(10, msg));
					finish();
					return;
				}
				else {

					if(inventory.hasPurchase(productId)){
						
						log("success, user has product: " + productId);
						log("IabPurchase finished: " + result + ", purchase: " + inventory.getPurchase(productId));
						String msg = inventory.getPurchase(productId).toString().replace("PurchaseInfo(type:inapp):", "");
						log(msg);
						UnityPlayer.UnitySendMessage(GameObjectName, "getInventoryResult", returnResult(0, msg));
						
						//PurchaseInfo(type:inapp):{"orderId": "SpmylLeuHpqWO9rC", "purchaseToken": "SpmylLeuHpqWO9rC", "developerPayload": "YOUR_DEVELOPER_PAYLOAD", "packageName": "YOUR_PACKAGE_NAME", "purchaseState": 0, "purchaseTime": 1427814481707, "productId": "YOUR_PRODUCT_ID"}

					}else{
						String msg = "user has no product: " + productId;
						log(msg);
						UnityPlayer.UnitySendMessage(GameObjectName, "getInventoryResult", returnResult(11, msg));
					}

				}
				
				finish();
			}
		};
	}

	private static void log(String msg){
		UnityPlayer.UnitySendMessage(GameObjectName, "DebugLog", msg);
		Log.d(TAG, msg);
	}
	
	private static String returnResult(int errorCode, String data){
		try {
			JSONObject json = new JSONObject();
			json.put("errorCode", errorCode);
			json.put("data", data);
			
			return json.toString();
			
		} catch (JSONException e) {
			e.printStackTrace();
			return "";
		}
	}
}
